/*    
 *    getReadSequence.cpp		
 *    MapPER
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Yin Hu
 *
 *    Authors: Yin Hu
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#define UNIX

#ifdef UNIX
#include <fstream>
#include <cstring>
#include <iostream>
#include <cmath>
#include <cstdlib> 
#include <ctime>
#else
#include <fstream>
#include <string>
#include <iostream>
#include <math.h>
#include <stdlib.h> 
#include <time.h>
#endif


using namespace std;

char readSequenceBuffer[500];
char readNameBuffer[500];

class alignmentSequence
{
public:
	long alignmentID;
	char name[200];
	char sequence[200];
	alignmentSequence *next;

	alignmentSequence();
};

alignmentSequence::alignmentSequence(){alignmentID = 0; next = NULL;}


class readInfo
{
public:
	char ID[200];
	alignmentSequence *alignments;

	readInfo();
};

readInfo::readInfo(){alignments = NULL;}


const long MAX_READ_COUNT = 10000000;
readInfo* allRead[MAX_READ_COUNT];
long allReadCnt = 0;

long searchReadIndex = 1;

void inputSequence(char *inputfilename)
{
	//assume the info file has been sorted according to the name
	ifstream inputfile;
	inputfile.open(inputfilename);

	char name[200];
	alignmentSequence *newAlignment;
	string info;

	name[0] = '\0';
	inputfile >> name;
	while (name[0] != '\0')
	{
		if (allReadCnt >= MAX_READ_COUNT)
		{
			cout << "read array overflow..." << endl;
			exit(1);
		}

		if (allReadCnt < 1 || strcmp(allRead[allReadCnt]->ID, name) != 0)
		{
			allRead[++allReadCnt] = new readInfo;
			strcpy(allRead[allReadCnt]->ID, name);
		}

		newAlignment = new alignmentSequence;
		inputfile >> newAlignment->name;
		inputfile >> newAlignment->alignmentID;
		inputfile >> newAlignment->sequence;
		getline(inputfile, info);

		newAlignment->next = allRead[allReadCnt]->alignments;
		allRead[allReadCnt]->alignments = newAlignment;

		name[0] = '\0';
		inputfile >> name;
	}
	
	inputfile.close();
	return;
}

bool getSequence(char *readNm, long readID)
{
	alignmentSequence *curAlignment;
	long tmpIndex = searchReadIndex;

	while (tmpIndex <= allReadCnt && strcmp(allRead[tmpIndex]->ID, readNm) != 0)
	{
		tmpIndex++;
	}

	if (tmpIndex > allReadCnt)
	{
		//not found, search again, forward
		tmpIndex = searchReadIndex;

		while (tmpIndex >= 1 && strcmp(allRead[tmpIndex]->ID, readNm) != 0)
		{
			tmpIndex--;
		}

		if (tmpIndex < 1)
		{
			//not found...
			return false;
		} 
		else
		{
			//got it
			curAlignment = allRead[tmpIndex]->alignments;
			while (curAlignment != NULL)
			{
				if (curAlignment->alignmentID == readID)
				{
					strcpy(readNameBuffer, curAlignment->name);
					strcpy(readSequenceBuffer, curAlignment->sequence);
					searchReadIndex = tmpIndex;
					return true;
				}
				curAlignment = curAlignment->next;
			}
			return false;
		}
	} 
	else
	{
		//got it
		curAlignment = allRead[tmpIndex]->alignments;
		while (curAlignment != NULL)
		{
			if (curAlignment->alignmentID == readID)
			{
				strcpy(readNameBuffer, curAlignment->name);
				strcpy(readSequenceBuffer, curAlignment->sequence);
				searchReadIndex = tmpIndex;
				return true;
			}
			curAlignment = curAlignment->next;
		}
		return false;
	}
}

void parseSAM(char *inputfilename, char *outputfilename)
{
	//parse SAM file, add sequence information

	char name[200], strand[10], chm[50], quality[100], CIGAR[1000], mateRef[100], matePos[100], obsLen[100], sequence[2000], qualityAdd[100], tmpChar[1000], readname[200];
	long startPosition, readID_start, readID_end, pathLength, readLength, iLoop;
	string info;

	ifstream inputfile;
	inputfile.open(inputfilename);
	ofstream outputfile;
	outputfile.open(outputfilename);

	searchReadIndex = 1;

	name[0] = '\0';
	inputfile >> name;
	while (name[0] != '\0')
	{
		inputfile >> strand;
		inputfile >> chm;
		inputfile >> startPosition;
		inputfile >> quality;
		inputfile >> CIGAR;
		inputfile >> mateRef;
		inputfile >> matePos;
		inputfile >> obsLen;
		inputfile >> sequence;
		inputfile >> qualityAdd;
		inputfile >> readID_start;
		inputfile >> readID_end;
		inputfile >> pathLength;
		inputfile >> readLength;
		getline(inputfile, info);
		
		strcpy(readname, name);
		if (getSequence(name, readID_start) == true)
		{
			strcpy(sequence, readSequenceBuffer);
			strcpy(readname, readNameBuffer);
		}
		else
		{
			cout << "warning: read " << name << " cannot get its start read sequence" << endl;
			for (iLoop = 0; iLoop < readLength; iLoop++)
			{
				tmpChar[iLoop] = 'N';
			}
			tmpChar[iLoop] = '\0';
			strcpy(sequence, tmpChar);
		}
		for (iLoop = 0; iLoop < pathLength; iLoop++)
		{
			tmpChar[iLoop] = 'N';
		}
		tmpChar[iLoop] = '\0';
		strcat(sequence, tmpChar);
		if (getSequence(name, readID_end) == true)
		{
			strcat(sequence, readSequenceBuffer);
			strcpy(readname, readNameBuffer);
		}
		else
		{
			cout << "warning: read " << name << " cannot get its end read sequence" << endl;
			for (iLoop = 0; iLoop < readLength; iLoop++)
			{
				tmpChar[iLoop] = 'N';
			}
			tmpChar[iLoop] = '\0';
			strcat(sequence, tmpChar);
		}

		outputfile << readname << "\t" << strand << "\t" << chm << "\t" << startPosition << "\t" << quality	 << "\t" << CIGAR << "\t" << mateRef << "\t" << matePos << "\t" << obsLen << "\t" << 
			sequence << "\t" << qualityAdd << endl;

		name[0] = '\0';
		inputfile >> name;
	}

	inputfile.close();
	outputfile.close();

	return;
}

// void parseFusion(char *inputfilename, char *outputfilename)
// {
// 	//parse fusion file, add sequence information
// 
// 	char name[200], strand1[10], strand2[10], startRead[500], pathInfo[1000], endRead[500], sequence[2000], sequence2[2000], tmpChar[1000], readname[200];
// 	long readID_start, readID_end, pathLength, readLength, iLoop;
// 	string info;
// 
// 	ifstream inputfile;
// 	inputfile.open(inputfilename);
// 	ofstream outputfile;
// 	outputfile.open(outputfilename);
// 
// 	searchReadIndex = 1;
// 
// 	name[0] = '\0';
// 	inputfile >> name;
// 	while (name[0] != '\0')
// 	{
// 		inputfile >> strand1;
// 		inputfile >> startRead;
// 		inputfile >> pathInfo;
// 		inputfile >> strand2;
// 		inputfile >> endRead;
// 		inputfile >> readID_start;
// 		inputfile >> readID_end;
// 		inputfile >> pathLength;
// 		inputfile >> readLength;
// 		getline(inputfile, info);
// 		
// 		strcpy(readname, name);
// 		if (getSequence(name, readID_start) == true)
// 		{
// 			strcpy(sequence, readSequenceBuffer);
// 			strcpy(readname, readNameBuffer);
// 		}
// 		else
// 		{
// 			cout << "warning: read " << name << " cannot get its start read sequence" << endl;
// 			for (iLoop = 0; iLoop < readLength; iLoop++)
// 			{
// 				tmpChar[iLoop] = 'N';
// 			}
// 			tmpChar[iLoop] = '\0';
// 			strcpy(sequence, tmpChar);
// 		}
// 		//do not output path sequence, because the path may be cross chromosome!!
// // 		for (iLoop = 0; iLoop < pathLength; iLoop++)
// // 		{
// // 			tmpChar[iLoop] = 'N';
// // 		}
// // 		tmpChar[iLoop] = '\0';
// //		strcat(sequence, tmpChar);
// 		if (getSequence(name, readID_end) == true)
// 		{
// 			strcpy(sequence2, readSequenceBuffer);
// 			strcpy(readname, readNameBuffer);
// 		}
// 		else
// 		{
// 			cout << "warning: read " << name << " cannot get its end read sequence" << endl;
// 			for (iLoop = 0; iLoop < readLength; iLoop++)
// 			{
// 				tmpChar[iLoop] = 'N';
// 			}
// 			tmpChar[iLoop] = '\0';
// 			strcpy(sequence2, tmpChar);
// 		}
// 
// 		outputfile << readname << "\t" << strand1 << "\t" << startRead << "\t" << pathInfo << "\t" << strand2 << "\t" << endRead << "\t" << sequence << "\t" << sequence2 << endl;
// 
// 		name[0] = '\0';
// 		inputfile >> name;
// 	}
// 
// 	inputfile.close();
// 	outputfile.close();
// 
// 	return;
// }


void parseFusion(char *inputfilename, char *outputfilename)
{
	//parse fusion file, add sequence information
	//also parse the path

	char name[200], pathInfo[1000], sequence[2000], sequence2[2000], tmpChar[1000], readname[200];
	char strand_start_seg1[10], strand_start_seg2[10], chr_start_seg1[10], chr_start_seg2[10], startRead_seg1[500], startRead_seg2[500];
	char strand_end_seg1[10], strand_end_seg2[10], chr_end_seg1[10], chr_end_seg2[10], endRead_seg1[500], endRead_seg2[500];	
	int isfusion_start, isfusion_end;
	long readID_start, readID_end, pathLength, readLength, iLoop, startPos_start_seg1, startPos_start_seg2, startPos_end_seg1, startPos_end_seg2;
	string info;

	ifstream inputfile;
	inputfile.open(inputfilename);
	ofstream outputfile;
	outputfile.open(outputfilename, fstream::app);

	searchReadIndex = 1;

	name[0] = '\0';
	inputfile >> name;
	while (name[0] != '\0')
	{
		inputfile >> readID_start;
		inputfile >> readID_end;
		inputfile >> pathLength;
		inputfile >> readLength;

		inputfile >> isfusion_start;
		if (isfusion_start > 0)
		{
			inputfile >> strand_start_seg1;
			inputfile >> chr_start_seg1;
			inputfile >> startPos_start_seg1;
			inputfile >> startRead_seg1;

			inputfile >> strand_start_seg2;
			inputfile >> chr_start_seg2;
			inputfile >> startPos_start_seg2;
			inputfile >> startRead_seg2;
		} 
		else
		{
			inputfile >> strand_start_seg1;
			inputfile >> chr_start_seg1;
			inputfile >> startPos_start_seg1;
			inputfile >> startRead_seg1;
		}

		inputfile >> pathInfo;

		inputfile >> isfusion_end;
		if (isfusion_end > 0)
		{
			inputfile >> strand_end_seg1;
			inputfile >> chr_end_seg1;
			inputfile >> startPos_end_seg1;
			inputfile >> endRead_seg1;

			inputfile >> strand_end_seg2;
			inputfile >> chr_end_seg2;
			inputfile >> startPos_end_seg2;
			inputfile >> endRead_seg2;
		} 
		else
		{
			inputfile >> strand_end_seg1;
			inputfile >> chr_end_seg1;
			inputfile >> startPos_end_seg1;
			inputfile >> endRead_seg1;
		}
		getline(inputfile, info);

		strcpy(readname, name);
		if (getSequence(name, readID_start) == true)
		{
			strcpy(sequence, readSequenceBuffer);
			strcpy(readname, readNameBuffer);
		}
		else
		{
			cout << "warning: read " << name << " cannot get its start read sequence" << endl;
			for (iLoop = 0; iLoop < readLength; iLoop++)
			{
				tmpChar[iLoop] = 'N';
			}
			tmpChar[iLoop] = '\0';
			strcpy(sequence, tmpChar);
		}
		//do not output path sequence, because the path may be cross chromosome!!
		// 		for (iLoop = 0; iLoop < pathLength; iLoop++)
		// 		{
		// 			tmpChar[iLoop] = 'N';
		// 		}
		// 		tmpChar[iLoop] = '\0';
		//		strcat(sequence, tmpChar);
		if (getSequence(name, readID_end) == true)
		{
			strcpy(sequence2, readSequenceBuffer);
			strcpy(readname, readNameBuffer);
		}
		else
		{
			cout << "warning: read " << name << " cannot get its end read sequence" << endl;
			for (iLoop = 0; iLoop < readLength; iLoop++)
			{
				tmpChar[iLoop] = 'N';
			}
			tmpChar[iLoop] = '\0';
			strcpy(sequence2, tmpChar);
		}

		outputfile << readname << "\t";
		if (isfusion_start > 0)
		{
			outputfile << strand_start_seg1 << strand_start_seg2 << "\t" << chr_start_seg1 << "_" << chr_start_seg2 << "\t" << startPos_start_seg1 << "_" << startRead_seg1
				 << "//" << startPos_start_seg2 << "_" << startRead_seg2;
		} 
		else
		{
			outputfile << strand_start_seg1 << "\t" << chr_start_seg1 << "\t" << startPos_start_seg1 << "_" << startRead_seg1;
		}
		outputfile << "\t" << pathInfo << "\t";

		if (isfusion_end > 0)
		{
			outputfile << strand_end_seg1 << strand_end_seg2 << "\t" << chr_end_seg1 << "_" << chr_end_seg2 << "\t" << startPos_end_seg1 << "_" << endRead_seg1
				 << "//" << startPos_end_seg2 << "_" << endRead_seg2;
		} 
		else
		{
			outputfile << strand_end_seg1 << "\t" << chr_end_seg1 << "\t" << startPos_end_seg1 << "_" << endRead_seg1;
		}
		outputfile << "\t" << sequence << "\t" << sequence2 << endl;

		name[0] = '\0';
		inputfile >> name;
	}

	inputfile.close();
	outputfile.close();

	return;
}


int main(int argc, char* argv[])
{
	char comd[1000], targetChr[100], inputfilename[1000], outputfilename[1000], startChr[100], endChr[100], resultPath[1000];
#ifdef UNIX
	if (argc != 4)
	{
		cout << argv[0] << "\t<chr#>\t<chr#>\t<targetpath>" << endl;
		return 1;
	}
	strcpy(startChr, argv[1]);
	strcpy(endChr, argv[2]);
	strcpy(resultPath, argv[3]);

	sprintf(targetChr, "%s%s", startChr, endChr);

	sprintf(comd, "sort -n +0 -1 %sdata/PER/%s.info > %sdata/PER/%s_sorted.info", resultPath, targetChr, resultPath, targetChr);
	system(comd);	
	sprintf(inputfilename, "%sdata/PER/%s_sorted.info", resultPath, targetChr);
	inputSequence(inputfilename);
	sprintf(comd, "rm %sdata/PER/%s_sorted.info", resultPath, targetChr);
	system(comd);	

//	cout << allReadCnt << endl;

	if (strcmp(startChr, endChr) == 0)
	{
		sprintf(comd, "sort -n +0 -1 %sresult/PER_prob/PERprob_%s.normal> %sresult/PER_prob/PERprob_%s_sorted.normal", resultPath, targetChr, resultPath, targetChr);
		system(comd);	
		sprintf(inputfilename, "%sresult/PER_prob/PERprob_%s_sorted.normal", resultPath, targetChr);
		sprintf(outputfilename, "%sresult/PER_prob/PERprob_%s.sam", resultPath, targetChr);
		parseSAM(inputfilename, outputfilename);
		sprintf(comd, "rm %sresult/PER_prob/PERprob_%s.normal", resultPath, targetChr);
		system(comd);	
		sprintf(comd, "rm %sresult/PER_prob/PERprob_%s_sorted.normal", resultPath, targetChr);
		system(comd);	
	}

//	cout << "SAM done" << endl;

	sprintf(comd, "sort -n +0 -1 %sresult/fusionRead/fusionRead_%s.fusion > %sresult/fusionRead/fusionRead_%s_sorted.fusion", resultPath, targetChr, resultPath, targetChr);
	system(comd);	
	sprintf(inputfilename, "%sresult/fusionRead/fusionRead_%s_sorted.fusion", resultPath, targetChr);
	sprintf(outputfilename, "%sresult/PER_prob/fusionRead.fusion", resultPath);
	parseFusion(inputfilename, outputfilename);
// 	sprintf(comd, "rm %sresult/PER_prob/fusionRead_%s.fusion", resultPath, targetChr);
// 	system(comd);	
	sprintf(comd, "rm %sresult/fusionRead/fusionRead_%s_sorted.fusion", resultPath, targetChr);
	system(comd);	

//	cout << "Fusion done" << endl;
#else
	sprintf(startChr, "chr1");
	sprintf(endChr, "chr1");
	sprintf(resultPath, "");

	sprintf(targetChr, "%s%s", startChr, endChr);

	sprintf(inputfilename, "%s%s_sorted.info", resultPath, targetChr);
	inputSequence(inputfilename);

	cout << allReadCnt << endl;

	if (strcmp(startChr, endChr) == 0)
	{
		sprintf(inputfilename, "%sPERprob_%s_sorted.sam", resultPath, targetChr);
		sprintf(outputfilename, "%sPERprob_%s_final.sam", resultPath, targetChr);
		parseSAM(inputfilename, outputfilename);
	}

	cout << "SAM done" << endl;

	sprintf(inputfilename, "%sfusionRead_%s_sorted.fusion", resultPath, targetChr);
	sprintf(outputfilename, "%sfusionRead_%s_final.fusion", resultPath, targetChr);
	parseFusion(inputfilename, outputfilename);

	cout << "Fusion done. Please confirm..." << endl;
	cin >> allReadCnt;
#endif	

	for (long iLoop = 1; iLoop <= allReadCnt; iLoop++)
	{
		delete allRead[iLoop];
	}
	
	return 0;
}
