/*    
 *    parsePER.cpp		
 *    MapPER
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Yin Hu
 *
 *    Authors: Yin Hu
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */










/************************************************************************/
/* Feb 22nd. ALL PER from start read to end read                        */
/************************************************************************/



#define UNIX

#ifdef UNIX
#include <fstream>
#include <cstring>
#include <iostream>
#include <cmath>
#include <cstdlib> 
#include <ctime>
#else
#include <fstream>
#include <string>
#include <iostream>
#include <math.h>
#include <stdlib.h> 
#include <time.h>
#endif


using namespace std;

const long MAX = 2000000000;
const int MAX_CHOICE_NUM = 100000;
const int MAX_COMB_LIMIT = 100;

const int MAXENDREADLENGTH = 500;

long PERCnt_same;
long PERCnt_diff;

char targetChromosome[1000][50];
int targetChrNum;
ofstream outputfile_chrname;
ofstream dropped_reads;
ofstream abnormal_reads;
ofstream overlap_reads;

char dirPrefix[500];
char curChromosomeName[50];

class reads
{
public:
	char name[50];
	char readName[100];
	long endID;
	bool fusionRead;
	int chrNum1; //chromosome number
	int chrNum2;
	char chromosome1[10];
	char chromosome2[10];
	char strand1;
	char strand2;
	long startpoint;
	long endpoint;
	long startpoint2;
	long endpoint2;
	char end[100];
	char end2[100];
	char CIGAR[MAXENDREADLENGTH];

	reads();
	void compute_endpoint();
	void compute_endpoint2();
	void compute_chrNum();
};

reads* startlist[MAX_CHOICE_NUM];
reads* endlist[MAX_CHOICE_NUM];
long startlist_count;
long endlist_count;
long endReadID;

ofstream allDistanceToFile;


reads::reads()
{
	fusionRead = false;
// 	for (int i = 0; i < 50; i++)
// 	{
// 		name[i] = '\0';
// 	}
// 	for (int i = 0; i < 10; i++)
// 	{
// 		chromosome1[i] = '\0';
// 		chromosome2[i] = '\0';
// 	}
// 	for (int i = 0; i < 100; i++)
// 	{
// 		end[i] = '\0';
// 		end2[i] = '\0';
// 	}
	strand1 = '+';
	strand2 = '-';
	endID = 0;
	chrNum1 = 0;
	chrNum2 = 0;
	startpoint = 0;
	startpoint2 = 0;
	endpoint = 0;
	endpoint2 = 0;
	
	name[0] = '\0';
	chromosome1[0] = '\0';
	chromosome2[0] = '\0';
	end[0] = '\0';
	end2[0] = '\0';
	CIGAR[0] = '\0';
}

void reads::compute_endpoint()
{
	int i, tmp = 0;

	endpoint = startpoint - 1;

	for (i = 0; end[i] != '\0'; i++)
	{
		if (end[i] == 'M')
		{
			endpoint = endpoint + tmp;
			tmp = 0;
		} 
		else if (end[i] == 'N')
		{
			endpoint = endpoint + tmp;
			tmp = 0;
		}
		else
		{
			tmp = tmp * 10 + end[i] - 48;
		}
	}

	return;
}


void reads::compute_endpoint2()
{
	int i, tmp = 0;

	endpoint2 = startpoint2 - 1;

	for (i = 0; end2[i] != '\0'; i++)
	{
		if (end2[i] == 'M')
		{
			endpoint2 = endpoint2 + tmp;
			tmp = 0;
		} 
		else if (end2[i] == 'N')
		{
			endpoint2 = endpoint2 + tmp;
			tmp = 0;
		}
		else
		{
			tmp = tmp * 10 + end2[i] - 48;
		}
	}

	return;
}



long compute_distance(reads* start_read, reads* end_read)
{
	//compute the min distance between the start_read and the end_read

//	return MAX;
	return end_read->startpoint - start_read->endpoint;
}


void reads::compute_chrNum()
{
	int tmp = 0;

	for (int i = 3; chromosome1[i] != '\0'; i++)
	{
		tmp = tmp * 10 + chromosome1[i] - 48;
	}

	chrNum1 = tmp;

	tmp = 0;

	for (int i = 3; chromosome2[i] != '\0'; i++)
	{
		tmp = tmp * 10 + chromosome2[i] - 48;
	}

	chrNum2 = tmp;

	return;
}




int outputPEreads(reads* start_read, reads* end_read)
{
	//output a PER
	char outpurfilename[500];
	char chromosome[50];
	int targetChrIndex, tmp;
	long junctionStart = -1, junctionEnd = -1, dist;
	ofstream outputfile;
	

	if (start_read->fusionRead == false && end_read->fusionRead == false)
	{
		if (start_read->chrNum1 < end_read->chrNum1)
		{
			strcpy(chromosome, start_read->chromosome1);
			strcat(chromosome, end_read->chromosome1);
		} 
		else if (start_read->chrNum1 > end_read->chrNum1)
		{
			strcpy(chromosome, end_read->chromosome1);
			strcat(chromosome, start_read->chromosome1);
		} 
		else
		{
			if (start_read->endpoint < end_read->startpoint || start_read->startpoint > end_read->endpoint)
			{
				strcpy(chromosome, start_read->chromosome1);
				strcat(chromosome, end_read->chromosome1);

				if (start_read->endpoint < end_read->startpoint)
				{
					dist = end_read->startpoint - start_read->endpoint - 1;
				} 
				else if (start_read->startpoint > end_read->endpoint)
				{
					dist = start_read->startpoint - end_read->endpoint - 1;
				}
				if (dist >= 0 && dist < 500)
				{
					allDistanceToFile << dist << endl;
				}
			}
			else
			{
				overlap_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl; 
				return -1;
			}
		}
	} 
	else
	{
		if (start_read->fusionRead == true && end_read->fusionRead == false)
		{
			if (start_read->chrNum1 < start_read->chrNum2)
			{
				if (end_read->chrNum1 == start_read->chrNum1 && end_read->endpoint < start_read->startpoint || end_read->chrNum1 == start_read->chrNum2 && end_read->endpoint < start_read->startpoint2)
				{
				}
				else if (end_read->chrNum1 == start_read->chrNum2 && end_read->startpoint > start_read->endpoint2 || end_read->chrNum1 == start_read->chrNum1 && end_read->startpoint > start_read->endpoint)
				{
				} 
				else
				{
					abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
					return -1;
				}

// 				if (start_read->strand1 == '+' && start_read->strand2 == '+')
// 				{
// 					junctionStart = start_read->endpoint;
// 					junctionEnd = start_read->startpoint2;
// 				}
// 				else if (start_read->strand1 == '+' && start_read->strand2 == '-')
// 				{
// 					junctionStart = start_read->endpoint;
// 					junctionEnd = start_read->endpoint2;
// 				}
// 				else if (start_read->strand1 == '-' && start_read->strand2 == '+')
// 				{
// 					junctionStart = start_read->startpoint;
// 					junctionEnd = start_read->startpoint2;
// 				}
// 				else if (start_read->strand1 == '-' && start_read->strand2 == '-')
// 				{
// 					junctionStart = start_read->startpoint;
// 					junctionEnd = start_read->endpoint2;
// 				}

				strcpy(chromosome, start_read->chromosome1);
				strcat(chromosome, start_read->chromosome2);
			} 
			else if (start_read->chrNum1 == start_read->chrNum2)// && start_read->endpoint < start_read->startpoint2)
			{
				if (end_read->chrNum1 <= start_read->chrNum1) // || (end_read->chrNum1 == start_read->chrNum1 && end_read->endpoint < start_read->startpoint))
				{
					strcpy(chromosome, end_read->chromosome1);
					strcat(chromosome, start_read->chromosome1);
				}
				else if (end_read->chrNum1 > start_read->chrNum2) // || (end_read->chrNum1 == start_read->chrNum2 && end_read->startpoint > start_read->endpoint2))
				{
					strcpy(chromosome, start_read->chromosome1);
					strcat(chromosome, end_read->chromosome1);
				} 
				else
				{
					abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
					return -1;
				}

// 				if (start_read->strand1 == '+' && start_read->strand2 == '+')
// 				{
// 					junctionStart = start_read->endpoint;
// 					junctionEnd = start_read->startpoint2;
// 				}
// 				else if (start_read->strand1 == '+' && start_read->strand2 == '-')
// 				{
// 					junctionStart = start_read->endpoint;
// 					junctionEnd = start_read->endpoint2;
// 				}
// 				else if (start_read->strand1 == '-' && start_read->strand2 == '+')
// 				{
// 					junctionStart = start_read->startpoint;
// 					junctionEnd = start_read->startpoint2;
// 				}
// 				else if (start_read->strand1 == '-' && start_read->strand2 == '-')
// 				{
// 					junctionStart = start_read->startpoint;
// 					junctionEnd = start_read->endpoint2;
// 				}
			}
			else if (start_read->chrNum1 > start_read->chrNum2)
			{
				if (end_read->chrNum1 == start_read->chrNum2 && end_read->endpoint < start_read->startpoint2 || end_read->chrNum1 == start_read->chrNum1 && end_read->endpoint < start_read->startpoint)
				{
				}
				else if (end_read->chrNum1 == start_read->chrNum1 && end_read->startpoint > start_read->endpoint || end_read->chrNum1 == start_read->chrNum2 && end_read->startpoint > start_read->endpoint2)
				{
				} 
				else
				{
					abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
					return -1;
				}

// 				if (start_read->strand1 == '+' && start_read->strand2 == '+')
// 				{
// 					junctionEnd = start_read->endpoint;
// 					junctionStart = start_read->startpoint2;
// 				}
// 				else if (start_read->strand1 == '+' && start_read->strand2 == '-')
// 				{
// 					junctionEnd = start_read->endpoint;
// 					junctionStart = start_read->endpoint2;
// 				}
// 				else if (start_read->strand1 == '-' && start_read->strand2 == '+')
// 				{
// 					junctionEnd = start_read->startpoint;
// 					junctionStart = start_read->startpoint2;
// 				}
// 				else if (start_read->strand1 == '-' && start_read->strand2 == '-')
// 				{
// 					junctionEnd = start_read->startpoint;
// 					junctionStart = start_read->endpoint2;
// 				}

				strcpy(chromosome, start_read->chromosome2);
				strcat(chromosome, start_read->chromosome1);
			}
			else
			{
				abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
				return -1;
			}

			if (start_read->strand1 == '+' && start_read->strand2 == '+')
			{
				junctionStart = start_read->endpoint;
				junctionEnd = start_read->startpoint2;
			}
			else if (start_read->strand1 == '+' && start_read->strand2 == '-')
			{
				junctionStart = start_read->endpoint;
				junctionEnd = start_read->endpoint2;
			}
			else if (start_read->strand1 == '-' && start_read->strand2 == '+')
			{
				junctionStart = start_read->startpoint;
				junctionEnd = start_read->startpoint2;
			}
			else if (start_read->strand1 == '-' && start_read->strand2 == '-')
			{
				junctionStart = start_read->startpoint;
				junctionEnd = start_read->endpoint2;
			}
		}
		else if (start_read->fusionRead == false && end_read->fusionRead == true)
		{
			if (end_read->chrNum1 < end_read->chrNum2)
			{
				if (start_read->chrNum1 == end_read->chrNum1 && start_read->endpoint < end_read->startpoint || start_read->chrNum1 == end_read->chrNum2 && start_read->endpoint < end_read->startpoint2)
				{
				}
				else if (start_read->chrNum1 == end_read->chrNum2 && start_read->startpoint > end_read->endpoint2 || start_read->chrNum1 == end_read->chrNum1 && start_read->startpoint > end_read->endpoint)
				{
				} 
				else
				{
					abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
					return -1;
				}

// 				if (end_read->strand1 == '+' && end_read->strand2 == '+')
// 				{
// 					junctionStart = end_read->endpoint;
// 					junctionEnd = end_read->startpoint2;
// 				}
// 				else if (end_read->strand1 == '+' && end_read->strand2 == '-')
// 				{
// 					junctionStart = end_read->endpoint;
// 					junctionEnd = end_read->endpoint2;
// 				}
// 				else if (end_read->strand1 == '-' && end_read->strand2 == '+')
// 				{
// 					junctionStart = end_read->startpoint;
// 					junctionEnd = end_read->startpoint2;
// 				}
// 				else if (end_read->strand1 == '-' && end_read->strand2 == '-')
// 				{
// 					junctionStart = end_read->startpoint;
// 					junctionEnd = end_read->endpoint2;
// 				}

				strcpy(chromosome, end_read->chromosome1);
				strcat(chromosome, end_read->chromosome2);
			} 
			else if (end_read->chrNum1 == end_read->chrNum2)// && end_read->endpoint < end_read->startpoint2)
			{
				if (start_read->chrNum1 <= end_read->chrNum1) // || (start_read->chrNum1 == end_read->chrNum1 && start_read->endpoint < end_read->startpoint))
				{
					strcpy(chromosome, start_read->chromosome1);
					strcat(chromosome, end_read->chromosome1);
				}
				else if (start_read->chrNum1 > end_read->chrNum2) // || (start_read->chrNum1 == end_read->chrNum2 && start_read->startpoint > end_read->endpoint2))
				{
					strcpy(chromosome, end_read->chromosome1);
					strcat(chromosome, start_read->chromosome1);
				} 
				else
				{
					abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
					return -1;
				}

// 				if (end_read->strand1 == '+' && end_read->strand2 == '+')
// 				{
// 					junctionStart = end_read->endpoint;
// 					junctionEnd = end_read->startpoint2;
// 				}
// 				else if (end_read->strand1 == '+' && end_read->strand2 == '-')
// 				{
// 					junctionStart = end_read->endpoint;
// 					junctionEnd = end_read->endpoint2;
// 				}
// 				else if (end_read->strand1 == '-' && end_read->strand2 == '+')
// 				{
// 					junctionStart = end_read->startpoint;
// 					junctionEnd = end_read->startpoint2;
// 				}
// 				else if (end_read->strand1 == '-' && end_read->strand2 == '-')
// 				{
// 					junctionStart = end_read->startpoint;
// 					junctionEnd = end_read->endpoint2;
// 				}
			}
			else if (end_read->chrNum1 > end_read->chrNum2)
			{
				if (start_read->chrNum1 == end_read->chrNum2 && start_read->endpoint < end_read->startpoint2 || start_read->chrNum1 == end_read->chrNum1 && start_read->endpoint < end_read->startpoint)
				{
				}
				else if (start_read->chrNum1 == end_read->chrNum1 && start_read->startpoint > end_read->endpoint || start_read->chrNum1 == end_read->chrNum2 && start_read->startpoint > end_read->endpoint2)
				{
				} 
				else
				{
					abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
					return -1;
				}

// 				if (end_read->strand1 == '+' && end_read->strand2 == '+')
// 				{
// 					junctionEnd = end_read->endpoint;
// 					junctionStart = end_read->startpoint2;
// 				}
// 				else if (end_read->strand1 == '+' && end_read->strand2 == '-')
// 				{
// 					junctionEnd = end_read->endpoint;
// 					junctionStart = end_read->endpoint2;
// 				}
// 				else if (end_read->strand1 == '-' && end_read->strand2 == '+')
// 				{
// 					junctionEnd = end_read->startpoint;
// 					junctionStart = end_read->startpoint2;
// 				}
// 				else if (end_read->strand1 == '-' && end_read->strand2 == '-')
// 				{
// 					junctionEnd = end_read->startpoint;
// 					junctionStart = end_read->endpoint2;
// 				}

				strcpy(chromosome, end_read->chromosome2);
				strcat(chromosome, end_read->chromosome1);
			}
			else
			{
				abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
				return -1;
			}

			if (end_read->strand1 == '+' && end_read->strand2 == '+')
			{
				junctionStart = end_read->endpoint;
				junctionEnd = end_read->startpoint2;
			}
			else if (end_read->strand1 == '+' && end_read->strand2 == '-')
			{
				junctionStart = end_read->endpoint;
				junctionEnd = end_read->endpoint2;
			}
			else if (end_read->strand1 == '-' && end_read->strand2 == '+')
			{
				junctionStart = end_read->startpoint;
				junctionEnd = end_read->startpoint2;
			}
			else if (end_read->strand1 == '-' && end_read->strand2 == '-')
			{
				junctionStart = end_read->startpoint;
				junctionEnd = end_read->endpoint2;
			}
		} 
		else
		{
			abnormal_reads << start_read->name << '\t' << start_read->chromosome1 << '\t' << start_read->fusionRead << '\t' << start_read->startpoint << '\t' << start_read->end << '\t' << end_read->chromosome1 << '\t' << end_read->fusionRead << '\t' << end_read->startpoint << '\t' << end_read->end << endl;  
			return -1;
		}
	}


	for (tmp = 1; tmp <= targetChrNum; tmp++)
	{
		if (strcmp(targetChromosome[tmp], chromosome) == 0)
		{
			break;
		}
	}

	if (tmp > targetChrNum)
	{
		//chromosome not found
		outputfile_chrname << chromosome << endl;

		targetChrNum++;
		strcpy(targetChromosome[targetChrNum], chromosome);	
	}
	targetChrIndex = tmp;

#ifdef UNIX
	sprintf(outpurfilename, "%sdata/PER/%s.per", dirPrefix, chromosome);
#else
	sprintf(outpurfilename, "%s.per", chromosome);
#endif

	outputfile.open(outpurfilename, fstream::app);

	strcpy(curChromosomeName, chromosome);

	outputfile << start_read->name << "\t"; 

	if (start_read->fusionRead == false && end_read->fusionRead == false)
	{
		outputfile << start_read->endID << "\t0\t" << start_read->chromosome1 << "\t" << start_read->strand1 << "\t" << start_read->startpoint << "\t" << start_read->end << "\t";
		outputfile << end_read->endID   << "\t0\t" << end_read->chromosome1 << "\t" << end_read->strand1 << "\t" << end_read->startpoint << "\t" << end_read->end << endl;
	} 
	else
	{
		if (start_read->fusionRead == true && end_read->fusionRead == false)
		{
			outputfile << start_read->endID << "\t1\t" << start_read->chromosome1 << "\t" << start_read->startpoint << "\t" << start_read->end << '\t' << start_read->chromosome2 << '\t' << start_read->startpoint2 << "\t" << start_read->end2 << "\t" << start_read->strand1 << start_read->strand2 << "\t" << junctionStart << '\t' << junctionEnd << '\t';

			outputfile << end_read->endID   << "\t0\t" << end_read->chromosome1 << "\t" << end_read->strand1 << "\t" << end_read->startpoint << "\t" << end_read->end << endl;
		}
		else if (start_read->fusionRead == false && end_read->fusionRead == true)
		{
			outputfile << start_read->endID << "\t0\t" << start_read->chromosome1 << "\t" << start_read->strand1 << "\t" << start_read->startpoint << "\t" << start_read->end << "\t";

			outputfile << end_read->endID   << "\t1\t" << end_read->chromosome1 << "\t" << end_read->startpoint << "\t" << end_read->end << '\t' << end_read->chromosome2 << '\t' << end_read->startpoint2 << "\t" << end_read->end2 << "\t" << end_read->strand1 << end_read->strand2 << "\t" << junctionStart << '\t' << junctionEnd << endl;
		}
		else if (start_read->fusionRead == true && end_read->fusionRead == true)
		{
			outputfile << start_read->endID << "\t1\t" << start_read->chromosome1 << "\t" << start_read->startpoint << "\t" << start_read->end << '\t' << start_read->chromosome2 << '\t' << start_read->startpoint2 << "\t" << start_read->end2 << "\t" << start_read->strand1 << start_read->strand2 << "\t" << junctionStart << '\t' << junctionEnd << '\t';

			outputfile << end_read->endID   << "\t1\t" << end_read->chromosome1 << "\t" << end_read->startpoint << "\t" << end_read->end << '\t' << end_read->chromosome2 << '\t' << end_read->startpoint2 << "\t" << end_read->end2 << "\t" << end_read->strand1 << end_read->strand2 << "\t" << junctionStart << '\t' << junctionEnd << endl;
		}
	}
	outputfile.close();

	return targetChrIndex;
}


void outputPEfile()
{
	//extract all possible alignments

	long i, j, ii, jj, targetChrIndex = -1;
	bool isSame = false;
	ofstream outputfile;
	char outputfilename[1000];

	//CORRECT THE CRITERIA FOR OUTPUT
	//NOTICE FUSION

	if (startlist_count * endlist_count < MAX_COMB_LIMIT)
	{
		//extract all
		for (i = 1; i <= startlist_count; i++)
		{
			for (j = 1; j <= endlist_count; j++)
			{
				//if ((startlist[i]->endpoint < endlist[j]->startpoint && endlist[j]->startpoint - startlist[i]->endpoint < 400000)|| (startlist[i]->startpoint > endlist[j]->endpoint && startlist[i]->startpoint - endlist[j]->endpoint < 400000))
				targetChrIndex = outputPEreads(startlist[i], endlist[j]);
				if (targetChrIndex >= 0)
				{
#ifdef UNIX
					sprintf(outputfilename, "%sdata/PER/%s_tmp.info", dirPrefix, curChromosomeName);
#else
					sprintf(outputfilename, "%s_tmp.info", curChromosomeName);
#endif
					outputfile.open(outputfilename, fstream::app);	
					outputfile << startlist[i]->name << "\t" << startlist[i]->readName << "\t" << startlist[i]->endID << "\t" << startlist[i]->CIGAR << endl;			
					outputfile << endlist[j]->name << "\t" << endlist[j]->readName << "\t" << endlist[j]->endID << "\t" << endlist[j]->CIGAR << endl;
					outputfile.close();
				}
				if (startlist[i]->fusionRead == false && endlist[j]->fusionRead == false && strcmp(startlist[i]->chromosome1, endlist[j]->chromosome1) == 0)
					isSame = true;
			}
		}
		if (isSame == true)
			PERCnt_same++;
		else
			PERCnt_diff++;

		return;
	} 
	else //if (startlist_count * endlist_count < MAX_COMB_LIMIT * 10)
	{
		for (i = 1; i <= startlist_count; i++)
		{
			for (j = 1; j <= endlist_count; j++)
			{
//				if (startlist[i]->fusionRead == true || endlist[j]->fusionRead == true || (startlist[i]->fusionRead == false && endlist[j]->fusionRead == false && strcmp(startlist[i]->chromosome1, endlist[j]->chromosome1) == 0))
				if (startlist[i]->fusionRead == false && endlist[j]->fusionRead == false && strcmp(startlist[i]->chromosome1, endlist[j]->chromosome1) == 0)
				{
					if ((startlist[i]->endpoint < endlist[j]->startpoint && endlist[j]->startpoint - startlist[i]->endpoint < 400000)|| (startlist[i]->startpoint > endlist[j]->endpoint && startlist[i]->startpoint - endlist[j]->endpoint < 400000))
					{
						targetChrIndex = outputPEreads(startlist[i], endlist[j]);
						if (targetChrIndex >= 0)
						{
#ifdef UNIX
							sprintf(outputfilename, "%sdata/PER/%s_tmp.info", dirPrefix, curChromosomeName);
#else
							sprintf(outputfilename, "%s_tmp.info", curChromosomeName);
#endif
							outputfile.open(outputfilename, fstream::app);	
							outputfile << startlist[i]->name << "\t" << startlist[i]->readName << "\t" << startlist[i]->endID << "\t" << startlist[i]->CIGAR << endl;			
							outputfile << endlist[j]->name << "\t" << endlist[j]->readName << "\t" << endlist[j]->endID << "\t" << endlist[j]->CIGAR << endl;
							outputfile.close();
						}
					}
					isSame = true;
				} 
				/*else
				{
					for (ii = 1; ii <= startlist_count; ii++)
					{
						dropped_reads << startlist[ii]->name << "\t1\t" << startlist[ii]->chromosome1 << '\t' << startlist[ii]->startpoint << '\t' << startlist[ii]->end << endl;
					}
					for (jj = 1; jj <= endlist_count; jj++)
					{
						dropped_reads << endlist[jj]->name << "\t2\t" << endlist[jj]->chromosome2 << '\t' << endlist[jj]->startpoint << '\t' << endlist[jj]->end << endl;
					}
				}*/
			}
		}
		if (isSame == true)
			PERCnt_same++;

/*		for (ii = 1; ii <= startlist_count; ii++)
		{
			dropped_reads << startlist[ii]->name << "\t1\t" << startlist[ii]->chromosome1 << '\t' << startlist[ii]->startpoint << '\t' << startlist[ii]->end << endl;
		}
		for (jj = 1; jj <= endlist_count; jj++)
		{
			dropped_reads << endlist[jj]->name << "\t2\t" << endlist[jj]->chromosome1 << '\t' << endlist[jj]->startpoint << '\t' << endlist[jj]->end << endl;
		}*/
	}

	return;
}

void cleanup()
{
	reads* ptr;

	for (long i = 1; i <= startlist_count; i++)
	{
		ptr = startlist[i];
		delete ptr;
	}
	for (long j = 1; j <= endlist_count; j++)
	{
		ptr = endlist[j];
		delete ptr;
	}

	startlist_count = 0;
	endlist_count = 0;

	endReadID = 0;

	return;
}

void parse(char* inputfilename)
{
	char outpurfilename[500];
	sprintf(outpurfilename, "%sdata/PER/ChromosomeName.txt", dirPrefix);
	outputfile_chrname.open(outpurfilename);
	sprintf(outpurfilename, "%sdata/PER/DroppedReads.txt", dirPrefix);
	dropped_reads.open(outpurfilename);
	sprintf(outpurfilename, "%sdata/PER/AbnormalReads.txt", dirPrefix);
	abnormal_reads.open(outpurfilename);
	sprintf(outpurfilename, "%sdata/PER/OverlapReads.txt", dirPrefix);
	overlap_reads.open(outpurfilename);

	reads *ptr;
	ptr = NULL;

	startlist_count = 0;
	endlist_count = 0;
	endReadID = 0;

	ifstream inputfile;
	inputfile.open(inputfilename);
		
	char name[200], cmd[1000], tmpname[200], readName[200];
	int start_end_switch, tmp, strand, tmp2;
	bool writeswitch;
	writeswitch = false;
	string info, curLine;
	long readIndexCount = 0;

	char field1[100], field2[100], field3[100], field4[100], field5[200], field6[100], field7[100], field8[100], field9[MAXENDREADLENGTH];

	for (tmp = 0; tmp < 100; tmp++)
	{
		name[tmp] = '\0';
	}

	inputfile >> name;


	while (name[0] != '\0')
	{
		//delete the read count before the read name!
		for (tmp = 0; tmp < 200; tmp++)
		{
			if (name[tmp] == '~' || name[tmp] == '\0')
			{
				break;
			}
		}
		if (name[tmp] == '~')
		{
			for (tmp++, tmp2 = 0; name[tmp] != '\0'; tmp++, tmp2++)
			{
				tmpname[tmp2] = name[tmp];
			}
			tmpname[tmp2] = '\0';
			strcpy(name, tmpname);
		}

		//find start_end_switch
		for (tmp = 0; tmp < 200; tmp++)
		{
			if (name[tmp] == '/' || name[tmp] == '\0')
			{
				break;
			}
		}
		if (name[tmp+1] == '1')
		{
			start_end_switch = 1;
		} 
		else
		{
			start_end_switch = 2;
		}

		name[tmp] = '\0';

		strcpy(readName, name);
		sprintf(name, "%ld", readIndexCount);

		inputfile >> field1;
		inputfile >> field2;
		inputfile >> field3;
		inputfile >> field4;
		inputfile >> field5;
		inputfile >> field6;
		inputfile >> field7;
		inputfile >> field8;
		inputfile >> field9;
		getline(inputfile, info);
		
		
	
		if (start_end_switch == 1)
		{
			if (writeswitch == true)
			{
				outputPEfile();
				cleanup();
				readIndexCount++;
				sprintf(name, "%ld", readIndexCount);
				writeswitch = false;
			}

			//start point
			startlist_count++;
			ptr = new reads;

			strcpy(ptr->name, name);
			strcpy(ptr->readName, readName);
			ptr->endID = (++endReadID);
			strcpy(ptr->CIGAR, field9);

			if (field6[0] == '=' || field6[0] == '*')
			{
				//not fusion read
				ptr->fusionRead = false;
				if (strcmp(field1, "16") != 0)
				{
					ptr->strand1 = '+';
				} 
				else
				{
					ptr->strand1 = '-';
				}

				strcpy(ptr->chromosome1, field2);
				ptr->startpoint = strtol(field3, NULL, 10);
				strcpy(ptr->end, field5);

				ptr->compute_endpoint();
			} 
			else
			{
				//fusion read
				ptr->fusionRead = true;
				strcpy(ptr->chromosome1, field1);
				ptr->strand1 = field2[0];
				ptr->startpoint = strtol(field3, NULL, 10);
				//ptr->endpoint = strtol(field4, NULL, 10);
				strcpy(ptr->end, field4);
				ptr->compute_endpoint();

				strcpy(ptr->chromosome2, field5);
				ptr->strand2 = field6[0];
				ptr->startpoint2 = strtol(field7, NULL, 10);
				//ptr->endpoint2 = strtol(field8, NULL, 10);
				strcpy(ptr->end2, field8);
				ptr->compute_endpoint2();
			}
			
			ptr->compute_chrNum();

			startlist[startlist_count] = ptr;
		} 
		else if (start_end_switch == 2)
		{
			//end point
			endlist_count++;
			ptr = new reads;

			strcpy(ptr->name, name);
			strcpy(ptr->readName, readName);
			ptr->endID = (++endReadID);
			strcpy(ptr->CIGAR, field9);

			if (field6[0] == '=' || field6[0] == '*')
			{
				//not fusion read
				ptr->fusionRead = false;
				if (strcmp(field1, "16") != 0)
				{
					ptr->strand1 = '+';
				} 
				else
				{
					ptr->strand1 = '-';
				}

				strcpy(ptr->chromosome1, field2);
				ptr->startpoint = strtol(field3, NULL, 10);
				strcpy(ptr->end, field5);

				ptr->compute_endpoint();
			} 
			else
			{
				//fusion read
				ptr->fusionRead = true;
				
				strcpy(ptr->chromosome1, field1);
				//ptr->strand1 = field2[0];
				if (field2[0] == '+')
				{
					ptr->strand1 = '+';
				} 
				else
				{
					ptr->strand1 = '-';
				}
				ptr->startpoint = strtol(field3, NULL, 10);
				//ptr->endpoint = strtol(field4, NULL, 10);
				strcpy(ptr->end, field4);
				ptr->compute_endpoint();

				strcpy(ptr->chromosome2, field5);
				//ptr->strand2 = field6[0];
				if (field6[0] == '+')
				{
					ptr->strand2 = '+';
				}
				else
				{
					ptr->strand2 = '-';
				}
				ptr->startpoint2 = strtol(field7, NULL, 10);
				//ptr->endpoint2 = strtol(field8, NULL, 10);
				strcpy(ptr->end2, field8);
				ptr->compute_endpoint2();
			}

			ptr->compute_chrNum();

			endlist[endlist_count] = ptr;

			writeswitch = true;
		}
	
		name[0] = '\0';
		inputfile >> name;		
	}

	outputfile_chrname << "PER Count on same chromosome\t" <<  PERCnt_same << endl << "PER Count on different chromosome\t" << PERCnt_diff << endl;

	inputfile.close();
	outputfile_chrname.close();
	dropped_reads.close();
	abnormal_reads.close();

	for (tmp = 1; tmp <= targetChrNum; tmp++)
	{
		sprintf(cmd, "sort -n +0 -1 %sdata/PER/%s_tmp.info > %sdata/PER/%s.info", dirPrefix, targetChromosome[tmp], dirPrefix, targetChromosome[tmp]);
		system(cmd);
		sprintf(cmd, "rm %sdata/PER/%s_tmp.info", dirPrefix, targetChromosome[tmp]);
		system(cmd);
	}

	return;
}



int main(int argc, char* argv[])
{
	targetChrNum = 0;
	PERCnt_same = 0;
	PERCnt_diff = 0;

#ifdef UNIX
	if (argc != 3)
	{
		cout << argv[0] << "\t<filename>\t<target_path>" << endl;
		return 1;
	}

	char inputfilename[500];

	strcpy(dirPrefix, argv[2]);
	sprintf(inputfilename, "%sdata/PER/matepair_distance.txt", dirPrefix);
	allDistanceToFile.open(inputfilename);

	strcpy(inputfilename, argv[1]);
	parse(inputfilename);

	allDistanceToFile.close();
#else
	parse("merged_paired.sam");
#endif

	return 0;
}
