/*    
 *    recaljunction.cpp		
 *    MapPER
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Yin Hu
 *
 *    Authors: Yin Hu
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#define UNIX

#ifdef UNIX
#include <fstream>
#include <cstring>
#include <iostream>
#include <cmath>
#include <cstdlib> 
#include <ctime>
#else
#include <fstream>
#include <string>
#include <iostream>
#include <math.h>
#include <stdlib.h> 
#include <time.h>
#endif

using namespace std;

class junction
{
public:
	long ID;
	char chr[10];
	int chrNum;
	int chrNum2;
	char strand[10];
	long startposition;
	long endposition;
	int isTrue;
	long pathSupp;
	long readSupp;

	junction *next;

	junction();
	void computeChrNum();
	void computeChrNum2();
};

junction::junction()
{
	ID = 0;
	chrNum = 0;
	chrNum2 = 0;
	startposition = 0;
	endposition = 0;
	isTrue = 1;
	pathSupp = 0;
	readSupp = 0;

	next = NULL;
}

void junction::computeChrNum()
{
	int tmp = 0, i = 3;

	for (; chr[i] != '\0' && chr[i] != '_'; i++)
	{
		tmp = tmp * 10 + chr[i] - 48;
	}

	chrNum = tmp;

	if (chr[i] == '_')
	{
		tmp = 0;
		for (i += 4; chr[i] != '\0'; i++)
		{
			tmp = tmp * 10 + chr[i] - 48;
		}

		chrNum2 = tmp;
	}
	else
		chrNum2 = 0;
	
	return;
}


const long MAX_CHR_LENGTH = 1000000000;
const long MAX_JUNC_NUM = 10000000;
junction* junctionList[MAX_JUNC_NUM];
long junctionListCnt = 0;
long sortKey_Junction[MAX_JUNC_NUM];

junction *allJunc = NULL;

long mergeSort_Larray[MAX_JUNC_NUM / 2 + 10];
long mergeSort_Rarray[MAX_JUNC_NUM / 2 + 10];
junction* mergeSort_LorderedList[MAX_JUNC_NUM / 2 + 10];
junction* mergeSort_RorderedList[MAX_JUNC_NUM / 2 + 10];

void merge_JunctionSort(long p, long q, long r)
{
	long n1, n2, i, j, k;

	n1 = q - p + 1;
	n2 = r - q;

	for (i = 1; i <= n1; i++)
	{
		mergeSort_Larray[i] = sortKey_Junction[p + i - 1];
		mergeSort_LorderedList[i] = junctionList[p + i - 1];
	}
	for (j = 1; j <= n2; j++)
	{
		mergeSort_Rarray[j] = sortKey_Junction[q + j];
		mergeSort_RorderedList[j] = junctionList[q + j];
	}

	mergeSort_Larray[n1 + 1] = MAX_CHR_LENGTH * 2;
	mergeSort_Rarray[n2 + 1] = MAX_CHR_LENGTH * 2;

	i = 1;
	j = 1;

	for (k = p; k <= r; k++)
	{
		if (mergeSort_Larray[i] <= mergeSort_Rarray[j])
		{
			sortKey_Junction[k] = mergeSort_Larray[i];
			junctionList[k] = mergeSort_LorderedList[i];

			i++;
		} 
		else
		{
			sortKey_Junction[k] = mergeSort_Rarray[j];
			junctionList[k] = mergeSort_RorderedList[j];

			j++;
		}
	}

	return;
}


void mergeSort_JunctionSort(long sortList_size)
{
	//non-recursive merge sort for sorting junctions
	long m, n, i, r;
	m = 1;
	n = sortList_size;

	while (m <= n)
	{
		i = 1;
		while (i <= n - m)
		{
			r = (i + 2 * m - 1) < n ? (i + 2 * m - 1) : n;
			merge_JunctionSort(i, i + m - 1, r);
			i = i + 2 * m;
		}

		m = m * 2;
	}

	return;
}



void inputjunction(char *inputfilename)
{
	ifstream inputfile;
	inputfile.open(inputfilename);

	long id;
	junction *newJunc;
	string info;

	id = -1;
	inputfile >> id;
	while (id >= 0)
	{
		newJunc = new junction;
		newJunc->ID = id;

		inputfile >> newJunc->chr;
		inputfile >> newJunc->strand;
		inputfile >> newJunc->startposition;
		inputfile >> newJunc->endposition;
		inputfile >> newJunc->isTrue;
		inputfile >> newJunc->pathSupp;
		inputfile >> newJunc->readSupp;
		getline(inputfile, info);

		junctionList[++junctionListCnt] = newJunc;

		id = -1;
		inputfile >> id;
	}

	inputfile.close();
	return;
}

void process(bool isfusion)
{
	if (junctionListCnt <= 1)
	{
		return;
	}

	long iLoop;
	junction *curJunc, *delJunc;
	allJunc = NULL;

	//sort...
	for (iLoop = 1; iLoop <= junctionListCnt; iLoop++)
	{
		sortKey_Junction[iLoop] = junctionList[iLoop]->endposition;
	}
	mergeSort_JunctionSort(junctionListCnt);
	for (iLoop = 1; iLoop <= junctionListCnt; iLoop++)
	{
		sortKey_Junction[iLoop] = junctionList[iLoop]->startposition;
	}
	mergeSort_JunctionSort(junctionListCnt);

	for (iLoop = 1; iLoop <= junctionListCnt; iLoop++)
	{
		junctionList[iLoop]->computeChrNum();
	}
	if (isfusion == true)
	{
		for (iLoop = 1; iLoop <= junctionListCnt; iLoop++)
		{
			sortKey_Junction[iLoop] = junctionList[iLoop]->chrNum2;
		}
		mergeSort_JunctionSort(junctionListCnt);
	}
	for (iLoop = 1; iLoop <= junctionListCnt; iLoop++)
	{
		sortKey_Junction[iLoop] = junctionList[iLoop]->chrNum;
	}
	mergeSort_JunctionSort(junctionListCnt);
	

	for (iLoop = junctionListCnt; iLoop >= 1; iLoop--)
	{
		junctionList[iLoop]->next = allJunc;
		allJunc = junctionList[iLoop];
	}

	curJunc = allJunc;
	while (curJunc->next != NULL)
	{
		if (strcmp(curJunc->chr, curJunc->next->chr) == 0 && strcmp(curJunc->strand, curJunc->next->strand) == 0 && curJunc->startposition == curJunc->next->startposition && curJunc->endposition == curJunc->next->endposition)
		{
			curJunc->isTrue = curJunc->isTrue > curJunc->next->isTrue? curJunc->isTrue : curJunc->next->isTrue;
			curJunc->pathSupp += curJunc->next->pathSupp;
			curJunc->readSupp += curJunc->next->readSupp;
			
			delJunc = curJunc->next;
			curJunc->next = curJunc->next->next;
			delete delJunc;
		}
		else
			curJunc = curJunc->next;		
	}

	return;
}

void outputjunction(char *outputfilename)
{
	ofstream outputfile;
	outputfile.open(outputfilename);

	junction *curJunc;

	curJunc = allJunc;
	while (curJunc != NULL)
	{
		outputfile << curJunc->ID << "\t" << curJunc->chr << "\t" << curJunc->strand << "\t" << curJunc->startposition << "\t" << curJunc->endposition
			<< "\t" << curJunc->isTrue << "\t" << curJunc->pathSupp << "\t" << curJunc->readSupp << endl;

		curJunc = curJunc->next;
	}

	outputfile.close();

	//clean up
	while (allJunc != NULL)
	{
		curJunc = allJunc;
		allJunc = allJunc->next;
		delete curJunc;
	}

	return;
}


int main(int argc, char* argv[])
{
	char inputfilename[1000], outputfilename[1000], comd[1000];

	//normal junctions
#ifdef UNIX
	if (argc != 2)
	{
		cout << argv[0] << "\t<file_path>" << endl;
		return 1;
	}
	
	sprintf(inputfilename, "%sresult/junction_support/splice.txt", argv[1]);
	sprintf(outputfilename, "%sresult/junction_support/splice.junction", argv[1]);
#else
	sprintf(inputfilename, "splice.txt");
	sprintf(outputfilename, "splice.junction");
#endif

	inputjunction(inputfilename);
	process(false);
	outputjunction(outputfilename);



	//fusion junctions
	junctionListCnt = 0;
#ifdef UNIX
	sprintf(inputfilename, "%sresult/junction_support/fusion.txt", argv[1]);
	sprintf(outputfilename, "%sresult/junction_support/fusion.junction", argv[1]);
#else
	sprintf(inputfilename, "fusion.txt");
	sprintf(outputfilename, "fusion.junction");
#endif

	inputjunction(inputfilename);
	process(true);
	outputjunction(outputfilename);

	sprintf(comd, "rm %sresult/junction_support/fusion.txt", argv[1]);
	system(comd);	
	sprintf(comd, "rm %sresult/junction_support/splice.txt", argv[1]);
	system(comd);	

	return 0;
}
