/*    
 *    check_input_files.cpp		
 *    MapSplice
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Kai Wang
 *
 *    Authors: Kai Wang
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <iostream>
#include <string>
#include <fstream>
#include <stdlib.h>
#include <errno.h>
using namespace std;

string
basename2(string filename) {

	//cout << "bef: "<<filename<<endl;
	const string s(filename.substr(0, filename.find_last_of(".")));
	size_t final_slash = s.find_last_of("/");

	if (final_slash == string::npos)
		final_slash = s.find_last_of("\\");
	if (final_slash != string::npos)
	{
		//cout << "aft 1: "<<s.substr(final_slash + 1)<<endl;
		return s.substr(final_slash + 1);
	}
	else
	{
		//cout << "aft 2: "<<s<<endl;
		return s;
	}
}

void convert_ex_fasta(char* extendFile)
{
	ifstream input_fs(extendFile);
	if( !input_fs ) 
	{
		fprintf(stderr,"error: open extended file error\n");exit(1);
	}

	string firstline;

	getline(input_fs, firstline);

	if (firstline.find(" ") != string::npos || firstline.find("\t") != string::npos)
	{
		cerr <<"chromosome name contain blank space or tab"<<endl;
		cerr << firstline << endl;
		exit(2);
	}

	string base_chr_file_name = basename2(extendFile);

	string base_chr_name = firstline.substr(1);

	if (base_chr_name != base_chr_file_name)
	{
		cerr << "base name of chromosome file not consistent with chromosome name" << endl;
		cerr << "base chromosome file name:" << base_chr_file_name << endl;
		cerr << "chromosome name:" <<base_chr_name << endl;
		exit(3);
	}

	input_fs.close();
}

int main(int argc, char* argv[])
{
	if (argc < 2 )
	{
		fprintf(stderr,"./check_input_files chromosome_file1 chromosome_file2 ...\n");exit(1);
	}

	for (int i = 1; i < argc; ++i)
		convert_ex_fasta(argv[i]);

	return 0;
}
