/*    
 *    dividereads.cpp		
 *    MapSplice
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Kai Wang
 *
 *    Authors: Kai Wang
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <stdio.h>
#include <iostream>
#include <string>
#include <fstream>
#include <stdlib.h>
using namespace std;

size_t minlen = 10000, maxlen = 0;

int countall=0;

void convert_ex_fasta(char* extendFile, ofstream& output_fs, ofstream& output_truncated_fs, int len)
{
	ifstream input_fs(extendFile);
	if( !input_fs ) 
	{
		fprintf(stderr,"error: open extended file error\n");exit(1);
	}    

	string readidline;
	string line;
	//int countall=0;
	while(!input_fs.eof())
	{
		getline(input_fs, readidline);
		getline(input_fs, line);
		if (line == "" || readidline == "")
			continue;
		countall++;

		//if (readidline[0] != '>')
		//	readidline[0] = '>';
//		>HWUSI-EAS524:2:1:1:82#0/1
//ANAATCAACACCCTCCTAGCCTTACTACTAATAATTATTACATTTTGACT

		//string firsthalf = line.substr(0, line.length()/2);

		if (line.length() > maxlen)
			maxlen = line.length();

		if (line.length() < minlen)
			minlen = line.length();
			

		if (len > line.length())
		{
			cerr << "some read length < "<<len << endl <<readidline << endl<< line<<endl;
			exit(2);
		}
		//int len = line.length() / num;

		int num = line.length() / len;

		for (int i = 1; i < num; ++i)
		{
			output_fs<< ">" <<i<<"~"<<readidline.c_str() + 1<<endl;
			output_fs<<line.substr((i-1)*len, len)<<endl;
		}

		//if (line.length() - (len*(num-1)) >= len)
		//{
		output_fs<<">" <<num<<"~" <<readidline.c_str() + 1<<endl;
		output_fs<<line.substr((num - 1)*len, len)<<endl;

		output_truncated_fs <<">" <<num + 1<<"~" <<readidline.c_str() + 1<<endl;
		output_truncated_fs <<line.substr(num*len)<<endl;
		//}
	}
	input_fs.close();
	
	cout<<countall<<" reads converted for "<<extendFile<<endl; 
}

void convert_ex_fastq(char* extendFile, ofstream& output_fs, ofstream& output_truncated_fs, int len)
{
	ifstream input_fs(extendFile);
	if( !input_fs ) 
	{
		fprintf(stderr,"error: open extended file error\n");exit(1);
	}

	string readidline;
	string line;
	string readidline2;
	string qualstr;
	//int countall=0;
	while(!input_fs.eof())
	{
		getline(input_fs, readidline);
		getline(input_fs, line);
		getline(input_fs, readidline2);
		getline(input_fs, qualstr);

		if (line == "" || readidline == "" || readidline2 == "" || qualstr == "")
			continue;

		countall++;

		if (line.length() > maxlen)
			maxlen = line.length();

		if (line.length() < minlen)
			minlen = line.length();

		//if (readidline[0] != '>')
		//	readidline[0] = '>';
//		>HWUSI-EAS524:2:1:1:82#0/1
//ANAATCAACACCCTCCTAGCCTTACTACTAATAATTATTACATTTTGACT

		//string firsthalf = line.substr(0, line.length()/2);

		//int len = line.length() / num;
		if (len > line.length() || len > qualstr.length())
		{
			cerr << "some read or quality length < "<<len << endl <<readidline << endl<< line<<endl << readidline2 <<endl<<qualstr << endl;
			exit(2);
		}

		int num = line.length() / len;

		for (int i = 1; i < num; ++i)
		{
			output_fs<<"@" <<i<<"~"<<readidline.c_str() + 1<<endl;
			output_fs<<line.substr((i-1)*len, len)<<endl;
			output_fs<<readidline2<<endl;
			output_fs<<qualstr.substr((i-1)*len, len)<<endl;
		}

		//if (line.length() - (len*(num-1)) >= len)
		//{
		output_fs<<"@" <<num<<"~"<<readidline.c_str() + 1<<endl;
		output_fs<<line.substr((num - 1)*len, len)<<endl;
		output_fs<<readidline2<<endl;
		output_fs<<qualstr.substr((num - 1)*len, len)<<endl;

		output_truncated_fs<<"@" <<num + 1<<"~"<<readidline.c_str() + 1<<endl;
		output_truncated_fs<<line.substr(num*len)<<endl;
		output_truncated_fs<<readidline2<<endl;
		output_truncated_fs<<qualstr.substr(num*len)<<endl;

		//}
	}
	input_fs.close();
	
	cout<<countall<<" reads converted for "<<extendFile<<endl; 
}

int main(int argc, char* argv[])
{
	if (argc < 5)
	{
		fprintf(stderr,"error: too few arguments\n");exit(1);
	}

	ofstream output_truncated_fs(argv[argc - 4]);

	ofstream output_fs(argv[argc-3]);

	int fasta = atoi(argv[argc - 2]);

	if( !output_fs ) 
	{
		fprintf(stderr,"error: write fasta file error\n");exit(1);
	}

	if (fasta)
	{
		for(int i=1;i<argc-4;i++)
		{
			convert_ex_fasta(argv[i], output_fs, output_truncated_fs, atoi(argv[argc-1]));
		}
	}
	else
	{
		for(int i=1;i<argc-4;i++)
		{
			convert_ex_fastq(argv[i], output_fs, output_truncated_fs, atoi(argv[argc-1]));
		}
	}

	cout << "minlen "<< minlen<<", maxlen "<<maxlen<<endl;

	output_fs.close();
	return 0;
}
