/*    
 *    pair_quality.h		
 *    MapSplice
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Zeng Zheng
 *
 *    Authors: Zeng Zheng
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef PAIR_QUALITY_H
#define PAIR_QUALITY_H

#include <string>
#include <math.h>
#include <algorithm> 
#include <stdlib.h>
using namespace std;
#pragma warning(disable:4996)

class Pair_Quality
{
public:
		int max_segment;
		int min_mate_dis;
		int min_mismatch;
		int min_intron;

		Pair_Quality()
		{
			max_segment= -1;
			min_mate_dis= -1;
			min_mismatch=-1;
			min_intron=-1;
		}

		Pair_Quality(const Sam_Info& sam1, const Sam_Info& sam2)
		{
			max_segment=abs((int)(sam1.end_seg_no - sam1.start_seg_no)) + abs((int)(sam2.end_seg_no - sam2.start_seg_no)) + 2;
			min_mate_dis=abs((int)(sam1.start_pos - sam2.start_pos));
			min_mismatch=(int)(sam1.mismatchs.size()+sam2.mismatchs.size());
			min_intron=0;
			for(size_t i=0;i<sam1.jump_code.size();i++)
			{
				if(sam1.jump_code[i].type == 'N')
					min_intron+=sam1.jump_code[i].len;
			}
			for(size_t i=0;i<sam2.jump_code.size();i++)
			{
				if(sam2.jump_code[i].type=='N')
					min_intron+=sam2.jump_code[i].len;
			}
		}

		void set_value(const Sam_Info& sam1, const Sam_Info& sam2)
		{
			max_segment=abs((int)(sam1.end_seg_no - sam1.start_seg_no)) + abs((int)(sam2.end_seg_no - sam2.start_seg_no)) + 2;
			min_mate_dis=abs((int)(sam1.start_pos - sam2.start_pos));
			min_mismatch=(int)(sam1.mismatchs.size()+sam2.mismatchs.size());
			min_intron=0;
			for(size_t i=0;i<sam1.jump_code.size();i++)
			{
				if(sam1.jump_code[i].type == 'N')
					min_intron+=sam1.jump_code[i].len;
			}
			for(size_t i=0;i<sam2.jump_code.size();i++)
			{
				if(sam2.jump_code[i].type=='N')
					min_intron+=sam2.jump_code[i].len;
			}
		}

		~Pair_Quality()
		{}

		void reset()
		{
			max_segment= -1;
			min_mate_dis= -1;
			min_mismatch=-1;
			min_intron=-1;
		}

		bool operator>(const Pair_Quality& new_pair) const 
		{  
			if(max_segment!=new_pair.max_segment)
				return max_segment > new_pair.max_segment;
			else if(min_mate_dis!=new_pair.min_mate_dis)
				return min_mate_dis < new_pair.min_mate_dis;
			else if(min_mismatch!= new_pair.min_mismatch)
				return min_mismatch < new_pair.min_mismatch;
			else
				return min_intron< new_pair.min_intron;
		}

		void operator=(const Pair_Quality& new_pair)
		{  
			max_segment = new_pair.max_segment;
			min_mate_dis = new_pair.min_mate_dis;
			min_mismatch = new_pair.min_mismatch;
			min_intron = new_pair.min_intron;
		}
};











#endif

