/*    
 *    pairing_fusion_normal_aligned.cpp		
 *    MapSplice
 *
 *    Copyright (C) 2010 University of Kentucky and
 *                       Kai Wang
 *
 *    Authors: Kai Wang
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define _CRT_SECURE_NO_WARNINGS 
//#pragma warning(disable:_CRT_SECURE_NO_WARNINGS)


#include <stdio.h>
#include <iostream>
#include <string>
#include <fstream>
#include <stdlib.h>
#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include <string.h>
#include <iterator>
using namespace std;



//#define VS

#ifdef VS
#include <hash_map> //vc only
#include <hash_set>
//#define _CRT_SECURE_NO_WARNINGS 
//#pragma warning(disable:_CRT_SECURE_NO_WARNINGS)
 
#else
#include <ext/hash_map> //g++ only
#include <ext/hash_set>

using __gnu_cxx::hash;
using __gnu_cxx::hash_map;
using __gnu_cxx::hash_set;

namespace __gnu_cxx
{
	template<typename Traits, typename Allocator>
	struct hash<std::basic_string<char, Traits, Allocator> >
	{
		size_t operator()(const std::basic_string<char, Traits, Allocator>& __s) const
		{
			return __stl_hash_string(__s.c_str());
		}
	};
}

#endif

#ifdef VS
using namespace stdext;
#endif

ofstream ofs_err;

struct PairedRegions
{
	string chrom1;

	size_t strand1;

	size_t start1, end1;

	hash_set<size_t> mapped_reads_index;

	string chrom2;

	size_t strand2;

	size_t start2, end2;

	//vector<size_t> mapped_reads_index2;	

	PairedRegions(const string& chr1, size_t sd1, size_t st1, size_t en1, const string& chr2,  size_t sd2, size_t st2, size_t en2) : chrom1(chr1), strand1(sd1), start1(st1), end1(en1), chrom2(chr2), strand2(sd2), start2(st2), end2(en2)
	{
	}

	string tostring()
	{
		char output_chr[5000];

		sprintf(output_chr, "%s\t%llu\t%llu\t%llu\t%s\t%llu\t%llu\t%llu", chrom1.c_str(), strand1, start1, end1, chrom2.c_str(), strand2, start2, end2);

		return output_chr;		
	}
};


typedef vector<PairedRegions> PAIRED_REGIONS_VEC;
typedef vector<PairedRegions>::iterator PAIRED_REGIONS_ITERATOR;
typedef PairedRegions* PAIRED_REGIONS_PTR;
typedef vector<PAIRED_REGIONS_PTR> PAIRED_REGIONS_PTR_VEC;
typedef vector<PAIRED_REGIONS_PTR>::iterator PAIRED_REGIONS_PTR_VEC_ITERATOR;
typedef hash_map<string, PAIRED_REGIONS_PTR_VEC > HASH_CHROM_PAIRED_REGIONS_PTR_VEC;
typedef hash_map<string, PAIRED_REGIONS_PTR_VEC >::iterator HASH_CHROM_PAIRED_REGIONS_PTR_VEC_ITERATOR;

struct SamRec {
	char tag_name[1000];
	unsigned short strand_t;
	char chrom_name[1000];
	size_t start, end;
	unsigned short confid;
	char splice_way[1000];
	char mapped_seq[1000];
	unsigned short mis_match;

	vector<pair<size_t, int> > spliceway_vec;

	bool wrong_format;

	size_t tagidx;

	double filter_score;

	size_t ave_intron_len;

	double ave_junc_mis;

	vector<string> junc_id;

	int best;

	char alters[1000];

	string alterstr;

	char qual_str[1000];

	bool isunique, isexonic, isspliced, issmallins, issmalldel;
	
	size_t canon_count, noncanon_count;

	double canon_rate;

	int matched_id;

	char mate_match;

	size_t mate_offest;

	int mate_diff;

	string cur_line;

	bool is_insert;

	size_t mappedlen;

	SamRec() {}

	SamRec(const string& line) : isunique(false), isexonic(false), isspliced(false), ave_intron_len(0), ave_junc_mis(-1), canon_count(0), noncanon_count(0), canon_rate(0), 
		issmallins(false), issmalldel(false), cur_line(line), is_insert(false), mappedlen(0)
	{

		size_t read_count = sscanf(line.c_str(), "%s\t%hu\t%s\t%llu\t%hu\t%s\t%c\t%llu\t%llu\t%s\t%s\tNM:i:%hu\t%s", 
			tag_name, &strand_t, chrom_name, &start, &confid, splice_way, &mate_match, &mate_offest, &mate_diff, mapped_seq, qual_str, &mis_match, alters);

		if (read_count == 13)
			alterstr = alters;

		size_t index = 0;

		string spliceway(splice_way);

		if ((spliceway.find("I") != string::npos))
			issmallins = true;
		else if ((spliceway.find("D") != string::npos))
			issmalldel = true;
		else if (spliceway.find("N") != string::npos)
			isspliced = true;
		else
			isexonic = true;		

		string flag_str = " ";

		while (true)
		{
			if (index >= spliceway.length())
				break;

			int maplen;

			char flag;

			sscanf(spliceway.c_str() + index, "%d%c", &maplen, &flag);

			if (flag_str[0] == ' ')
			{
				if (flag == 'I')
					spliceway_vec.push_back(make_pair(start, -maplen));
				else if (flag == 'M')
				{
					spliceway_vec.push_back(make_pair(start, maplen));

					mappedlen += maplen;
				}
				else if (flag == 'N')
				{
					cout<<"start with N?"<<endl;
					spliceway_vec.push_back(make_pair(start + maplen, 0));
				}
			}
			else if (flag_str[0] == 'M')
			{
				if (flag == 'I')
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first + spliceway_vec.back().second, -maplen));
				else if (flag == 'M')
				{
					cout << "continue Ms?"<<endl;
					spliceway_vec.back().second += maplen;

					mappedlen += maplen;
				}
				else if (flag == 'N')
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first + spliceway_vec.back().second + maplen, 0));
			}
			else if (flag_str[0] == 'N')
			{
				if (flag == 'I')
					spliceway_vec.back().second = -maplen;
				else if (flag == 'M')
				{
					spliceway_vec.back().second = maplen;

					mappedlen += maplen;
				}
				else if (flag == 'N')
				{
					cout << "continue Ns?"<<endl;
					spliceway_vec.back().first += maplen;
				}
			}
			else if (flag_str[0] == 'I')
			{
				if (flag == 'I')
				{
					cout << "continue Is?"<<endl;
					spliceway_vec.back().second += -maplen;
				}
				else if (flag == 'M')
				{
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first, maplen));

					mappedlen += maplen;
				}
				else if (flag == 'N')
					spliceway_vec.push_back(make_pair(spliceway_vec.back().first + maplen, 0));
			}

			flag_str[0] = flag;

			index = spliceway.find(flag_str, index) + 1;
		}

		end = spliceway_vec.back().first + spliceway_vec.back().second - 1;
	}

	string tostring()
	{
		if (!cur_line.empty())
			return cur_line;

		char sam_rec_char[5000];

		sprintf(sam_rec_char, "%s\t%hu\t%s\t%llu\t%hu\t%s\t%c\t%llu\t%d\t%s\t%s\tNM:i:%hu\t%s", tag_name, strand_t, 
			chrom_name, start, confid, splice_way, mate_match, mate_offest, mate_diff, mapped_seq, qual_str, mis_match, alterstr.c_str()/*, best, filter_score*/);

		return sam_rec_char;
	}
};

struct FusionRec{

	char tag_name[1000];

	char chrom_name_1[1000];

	char strand_1;

	size_t start1;

	size_t end1;

	char chrom_name_2[1000];

	char strand_2;

	size_t start2;

	size_t end2;

	char mapped_seq[1000];

	size_t mismatch;

	string cur_line;

	FusionRec(const string& line) : cur_line(line)
	{
		//30846~HWI-EAS217:4:1:236:220#0/2	chr7	+	75933144	75933181	chr9	-	74628402	74628413	CATGGCTACATCTCCCGGTGCTNCACGCGGAAATACACAAAAAAAAAAAA	GTAG	NM:i:2
		size_t read_count = sscanf(line.c_str(), "%s\t%s\t%c\t%llu\t%llu\t%s\t%c\t%llu\t%llu\t%s\tNM:i%llu", 
			tag_name, chrom_name_1, &strand_1, &start1, &end1, chrom_name_2, &strand_2, &start2, &end2, mapped_seq, &mismatch);
	}

	string tostring()
	{
		if (!cur_line.empty())
			return cur_line;

		char sam_rec_char[5000];

		sprintf(sam_rec_char, "%s\t%s\t%c\t%llu\t%llu\t%s\t%c\t%llu\t%llu\t%s\tNM:i%llu", tag_name, chrom_name_1, strand_1, start1, end1, chrom_name_2, strand_2, start2, end2, mapped_seq, mismatch);

		return sam_rec_char;
	}

	//string tostring(PAIRED_REGIONS_PTR_VEC_ITERATOR region_iter, vector<SamRec>::iterator sam_iter, int r_id)
	//{
	//	//if (!cur_line.empty())
	//	//	return cur_line;

	//	char sam_rec_char[5000];

	//	char mate_strand = '+';

	//	if (sam_iter->strand_t & 16)
	//		mate_strand = '-';

	//	size_t region_st, region_end;

	//	if (r_id == 1)
	//	{
	//		region_st = (*region_iter)->start2;

	//		region_end = (*region_iter)->end2;
	//	}
	//	else
	//	{
	//		region_st = (*region_iter)->start1;

	//		region_end = (*region_iter)->end1;
	//	}


	//	//3447~HWI-EAS217:4:1:39:1942#0/1	chr1	+	567820	1-3	CTCGGCCTATCCGGAATGCCCCGACGTTACTCGGACTACCCCGATGCATACACCACATGAAACATCCTATCATCT	GGAGGCTCATGCATTTCTCTAACAT,	
	//	//ZO[\Q_b_S_a[^a_PY[a`aaR]_ZU_T_aaaTXJ^_a`Z`_[^^XGZHQ[a_`a_YQUFY_PRXVYFW]SVYBBBBBBBBBBBBBBBBBBBBBBBBBB	75M	38:T>C,62:T>C,

	//	if (mate_strand == '+')
	//	{
	//		if (sam_iter->end > region_end)
	//		{
	//			sprintf(sam_rec_char, "%s\t%s\t%c\t%llu\t%s\t%c\t%llu\t%llu\t%hu-%hu\t%s\t%s\t%s\t%s\t%s", tag_name, chrom_name, strand, start, 
	//			sam_iter->chrom_name, mate_strand, sam_iter->end, region_end, start_id, end_id, mapped_seq, missed_seq, qual_seq, splice_way, alterstr.c_str());

	//			ofs_err << this->tostring() << endl << (*region_iter)->tostring()<<endl << sam_iter->tostring()<<endl<<sam_rec_char<<endl;
	//			sam_rec_char[0] = '\0';
	//		}
	//		else
	//			sprintf(sam_rec_char, "%s\t%s\t%c\t%llu\t%s\t%c\t%llu\t%llu\t%hu-%hu\t%s\t%s\t%s\t%s\t%s", tag_name, chrom_name, strand, start, 
	//			sam_iter->chrom_name, mate_strand, sam_iter->end, region_end, start_id, end_id, mapped_seq, missed_seq, qual_seq, splice_way, alterstr.c_str());


	//	}
	//	else if (mate_strand == '-')
	//	{
	//		if (region_st > sam_iter->start)
	//		{
	//			sprintf(sam_rec_char, "%s\t%s\t%c\t%llu\t%s\t%c\t%llu\t%llu\t%hu-%hu\t%s\t%s\t%s\t%s\t%s", tag_name, chrom_name, strand, start, 
	//			sam_iter->chrom_name, mate_strand, sam_iter->end, region_end, start_id, end_id, mapped_seq, missed_seq, qual_seq, splice_way, alterstr.c_str());

	//			ofs_err << this->tostring() << endl << (*region_iter)->tostring()<<endl << sam_iter->tostring()<<endl<<sam_rec_char<<endl;
	//			sam_rec_char[0] = '\0';
	//		}
	//		else
	//			sprintf(sam_rec_char, "%s\t%s\t%c\t%llu\t%s\t%c\t%llu\t%llu\t%hu-%hu\t%s\t%s\t%s\t%s\t%s", tag_name, chrom_name, strand, start, 
	//			sam_iter->chrom_name, mate_strand, region_st, sam_iter->start, start_id, end_id, mapped_seq, missed_seq, qual_seq, splice_way, alterstr.c_str());

	//		
	//	}

	//	return sam_rec_char;
	//}
};

template<typename T>
class ReadNextTagAlignHandler {

public:
	typedef T value_type;
	typedef size_t size_type;
	typedef T& reference;
	typedef const T& const_reference;

private:
	ifstream ifs;
	string prev_tag_name;
	string prev_line;
	string tag_name;

public:
	vector<value_type> stored_tag_aligns;
	int read_id;

public:
	ReadNextTagAlignHandler(const char* file_name)
	{
		ifs.open(file_name);

		if (!ifs.is_open())
		{
			cerr<<"can't open file: "<< file_name << endl;
			exit(1);
		}
	}

	size_t ReadNextTagAlign()
	{
		stored_tag_aligns.clear();

		tag_name = "";

		if (!prev_tag_name.empty())
		{
			tag_name = prev_tag_name;

			sscanf(tag_name.c_str(), "%d", &read_id);

			if (read_id % 2 == 0)
				read_id--;

			stored_tag_aligns.push_back(value_type(prev_line));
		}

		while (!ifs.eof() )
		{
			string line;

			getline(ifs, line);

			if (line.empty())
				continue;

			char tag_name_chr[1000];

			sscanf(line.c_str(), "%s", tag_name_chr);

			if (prev_tag_name.empty())
			{
				prev_tag_name = tag_name_chr;

				tag_name = prev_tag_name;

				sscanf(tag_name.c_str(), "%d", &read_id);

				if (read_id % 2 == 0)
					read_id--;

				stored_tag_aligns.push_back(value_type(line));
			}
			else if (prev_tag_name == tag_name_chr)
			{
				stored_tag_aligns.push_back(value_type(line));
			}
			else
			{
				prev_tag_name = tag_name_chr;

				prev_line = line;

				return stored_tag_aligns.size();
			}
		}

		prev_tag_name.clear();

		prev_line.clear();

		return stored_tag_aligns.size();
	}
};

inline char
complement(int i) {
	static const int b2c_size = 20;
	static const char b2c[] = {
		//A,  b,  C,  d,  e,  f,  g,  h,  i,  j,  k,  l,  m,  n,  o,  p,  q,  r,  s,  T
		'T','N','G','N','N','N','C','N','N','N','N','N','N','N','N','N','N','N','N','A'
	};
	static const char b2cl[] = {
		//A,  b,  C,  d,  e,  f,  g,  h,  i,  j,  k,  l,  m,  n,  o,  p,  q,  r,  s,  T
		't','n','g','n','n','n','c','n','n','n','n','n','n','n','n','n','n','n','n','a'
	};
	if (i - 'A' >= 0 && i - 'A' < b2c_size)
		return b2c[i - 'A'];
	else if (i - 'a' >= 0 && i - 'a' < b2c_size)
		return b2cl[i - 'a'];
	else return 'N';
}

inline string
revcomp(const string& s) {
	string r;
	transform(s.begin(), s.end(), back_inserter(r), complement);
	reverse(r.begin(), r.end());
	return r;
}



size_t max_dist = 0;

int ReadRegions(const char* region_file, PAIRED_REGIONS_VEC& paired_regions, size_t ext)
{
	ifstream ifs(region_file);

	int count = 0;

	if (ifs.is_open())
	{
		while (!ifs.eof() )
		{
			string line;
			getline(ifs,line);
			if (line.empty())
				continue;

			char chrom1[1000];

			size_t strand1;
			
			size_t start1, end1;

			char chrom2[1000];

			size_t strand2;

			size_t start2, end2;

			sscanf(line.c_str(), "%s\t%llu\t%llu\t%llu\t%s\t%llu\t%llu\t%llu", chrom1, &strand1, &start1, &end1, chrom2, &strand2, &start2, &end2);

			string chrom1str = chrom1;

			string chrom2str = chrom2;

			//if (chrom1str != "chr20" || chrom2str != "chr17")
			//	continue;

			if (start1 > ext)
				start1 -= ext;
			else
				start1 = 1;

			end1 += ext;

			if (start2 > ext)
				start2 -= ext;
			else
				start2 = 1;

			end2 += ext;

			paired_regions.push_back(PairedRegions(chrom1, strand1, start1, end1, chrom2, strand2, start2, end2));

			if (max_dist <  (end1 - start1))
				max_dist = end1 - start1;

			if (max_dist <  (end2 - start2))
				max_dist = end2 - start2;

			++count;
		}
	}
	else
		cout <<"can't open file: " << region_file << endl;

	return count;
}

bool comp1(const PairedRegions* lhs, const PairedRegions* rhs)
{
	if (lhs->start1 == rhs->start1)
		return lhs->end1 < rhs->end1 ;

	return lhs->start1 < rhs->start1;
}

bool comp2(const PairedRegions* lhs, const PairedRegions* rhs)
{
	if (lhs->start2 == rhs->start2)
		return lhs->end2 < rhs->end2;

	return lhs->start2 < rhs->start2;
}

void SortRegion(PAIRED_REGIONS_VEC& paired_regions, HASH_CHROM_PAIRED_REGIONS_PTR_VEC& paired_regions_sorted1, HASH_CHROM_PAIRED_REGIONS_PTR_VEC& paired_regions_sorted2)
{
	PAIRED_REGIONS_ITERATOR paired_regions_iter;

	for (paired_regions_iter = paired_regions.begin(); paired_regions_iter != paired_regions.end(); ++paired_regions_iter)
	{
		paired_regions_sorted1[paired_regions_iter->chrom1].push_back(&(*paired_regions_iter));

		paired_regions_sorted2[paired_regions_iter->chrom2].push_back(&(*paired_regions_iter));
	}

	HASH_CHROM_PAIRED_REGIONS_PTR_VEC_ITERATOR chrom_paired_regions_iter;
	for (chrom_paired_regions_iter = paired_regions_sorted1.begin(); chrom_paired_regions_iter != paired_regions_sorted1.end(); ++chrom_paired_regions_iter)
		sort(chrom_paired_regions_iter->second.begin(), chrom_paired_regions_iter->second.end(), comp1);

	for (chrom_paired_regions_iter = paired_regions_sorted2.begin(); chrom_paired_regions_iter != paired_regions_sorted2.end(); ++chrom_paired_regions_iter)
		sort(chrom_paired_regions_iter->second.begin(), chrom_paired_regions_iter->second.end(), comp2);
}

struct SamMatchedRegions {

	SamRec* cur_sam_rec_ptr;

	vector<PAIRED_REGIONS_PTR> PAIRED_REGIONS_PTR_VEC;
};

struct FusionMatchedRegions {

	FusionRec* cur_fusion_rec_ptr;

	vector<PAIRED_REGIONS_PTR> PAIRED_REGIONS_PTR_VEC;
};

int FilterFusionCandidateByRegions(const char* mapped_sam_file, const char* fusion_candidate_file, 
	hash_map<string, PAIRED_REGIONS_PTR_VEC >& paired_regions_sorted1, 
	hash_map<string, PAIRED_REGIONS_PTR_VEC >& paired_regions_sorted2)
{
	ifstream ifs_mapped_sam(mapped_sam_file);

	ifstream ifs_fusion_candidate(fusion_candidate_file);

	string fusion_candidate_file_paired(fusion_candidate_file); 
	fusion_candidate_file_paired.append(".paired"); 
	ofstream ofs_fusion_candidate_file_paired(fusion_candidate_file_paired.c_str());

	string fusion_candidate_file_fusion_single(fusion_candidate_file); 
	fusion_candidate_file_fusion_single.append(".fusion_single"); 
	ofstream ofs_fusion_candidate_fusion_single(fusion_candidate_file_fusion_single.c_str());

	string fusion_candidate_file_normal_single(fusion_candidate_file); 
	fusion_candidate_file_normal_single.append(".normal_single"); 
	ofstream ofs_fusion_candidate_normal_single(fusion_candidate_file_normal_single.c_str());

	int count = 0, matched_count1 = 0, matched_count2 = 0;

	ReadNextTagAlignHandler<SamRec> sam_file_handler(mapped_sam_file);

	ReadNextTagAlignHandler<FusionRec> fusion_file_handler(fusion_candidate_file);

	int sam_read_idx;

	int fusion_read_idx;

	size_t sam_count = sam_file_handler.ReadNextTagAlign();

	sam_read_idx = sam_file_handler.read_id;

	size_t fusion_count = fusion_file_handler.ReadNextTagAlign();

	fusion_read_idx = fusion_file_handler.read_id;

	while (true)
	{
		if (sam_count == 0 || fusion_count == 0)
			break;

		if (sam_read_idx == fusion_read_idx)//matched, start checking regions
		{
			vector<SamRec*> in_regions_sam_rec;
			vector<SamRec>::iterator sam_rec_iter;
			vector<SamRec> & sam_rec_vec = sam_file_handler.stored_tag_aligns;

			vector<FusionRec*> in_regions_fusion_rec;
			vector<FusionRec>::iterator fusion_rec_iter;
			vector<FusionRec> & fusion_rec_vec = fusion_file_handler.stored_tag_aligns;

			for (fusion_rec_iter = fusion_rec_vec.begin(); fusion_rec_iter !=fusion_rec_vec.end(); ++fusion_rec_iter)
			{
				bool remained = false;

				size_t start1 = fusion_rec_iter->start1;

				size_t end1 = fusion_rec_iter->end1;

				size_t strand1 = fusion_rec_iter->strand_1 == '+' ? 0 : 16;

				string chromname1 = fusion_rec_iter->chrom_name_1;

				size_t start2 = fusion_rec_iter->start2;

				size_t end2 = fusion_rec_iter->end2;

				size_t strand2 = fusion_rec_iter->strand_2 == '+' ? 0 : 16;

				string chromname2 = fusion_rec_iter->chrom_name_2;

				PairedRegions cur_region(chromname1, strand1, start1, end1, chromname2, strand2, start2, end2);

				PAIRED_REGIONS_PTR_VEC_ITERATOR upper_bound_iter1, upper_bound_iter2, upper_bound_prev_iter1, upper_bound_prev_iter2;

				upper_bound_iter1 = upper_bound(paired_regions_sorted1[chromname1].begin(), paired_regions_sorted1[chromname1].end(), &cur_region, comp1);

				while (upper_bound_iter1 != paired_regions_sorted1[chromname1].begin())
				{
					upper_bound_prev_iter1 = upper_bound_iter1 - 1;

					if ((*upper_bound_prev_iter1)->end1 + max_dist < cur_region.start1)
						break;

					if ((*upper_bound_prev_iter1)->start1 < cur_region.start1 && (*upper_bound_prev_iter1)->end1 > cur_region.start1) 
					{
						//search corresponding sam record
						for (sam_rec_iter = sam_rec_vec.begin(); sam_rec_iter != sam_rec_vec.end(); ++sam_rec_iter)
						{
							if ((*upper_bound_prev_iter1)->start2 < sam_rec_iter->start && (*upper_bound_prev_iter1)->end2 > sam_rec_iter->start)
							{
								ofs_fusion_candidate_file_paired<<fusion_rec_iter->tostring()<<endl;
								
								ofs_fusion_candidate_file_paired<<sam_rec_iter->tostring()<<endl;

								ofs_fusion_candidate_file_paired<<(*upper_bound_prev_iter1)->tostring()<<endl;

								//string formatted_str = fusion_rec_iter->tostring(upper_bound_prev_iter1, sam_rec_iter, 1);

								//if (formatted_str.length())
								//	if (fusion_rec_iter->start_id == seg_num || fusion_rec_iter->end_id == seg_num)
								//		ofs_fusion_candidate_file_remained_formated_2 << fusion_rec_iter->tostring(upper_bound_prev_iter1, sam_rec_iter, 1)<<endl;
								//	else
								//		ofs_fusion_candidate_file_remained_formated << fusion_rec_iter->tostring(upper_bound_prev_iter1, sam_rec_iter, 1)<<endl;

								remained = true;
							}
						}

						++matched_count1;
					}

					upper_bound_iter1 = upper_bound_prev_iter1;
				}

				upper_bound_iter2 = upper_bound(paired_regions_sorted2[chromname2].begin(), paired_regions_sorted2[chromname2].end(), &cur_region, comp2);

				while (upper_bound_iter2 != paired_regions_sorted2[chromname2].begin())
				{
					upper_bound_prev_iter2 = upper_bound_iter2 - 1;

					if ((*upper_bound_prev_iter2)->end2 + max_dist < cur_region.start1)
						break;

					if ((*upper_bound_prev_iter2)->start2 < cur_region.start1 && (*upper_bound_prev_iter2)->end2 > cur_region.start1)
					{
						//search corresponding sam record
						for (sam_rec_iter = sam_rec_vec.begin(); sam_rec_iter != sam_rec_vec.end(); ++sam_rec_iter)
						{
							if ((*upper_bound_prev_iter2)->start1 < sam_rec_iter->start && (*upper_bound_prev_iter2)->end1 > sam_rec_iter->start)
							{
								ofs_fusion_candidate_file_paired<<fusion_rec_iter->tostring()<<endl;
								ofs_fusion_candidate_file_paired<<(*upper_bound_prev_iter2)->tostring()<<endl;
								ofs_fusion_candidate_file_paired<<sam_rec_iter->tostring()<<endl<<endl;

								//string formatted_str = fusion_rec_iter->tostring(upper_bound_prev_iter2, sam_rec_iter, 2);

								//if (formatted_str.length())
								//	if (fusion_rec_iter->start_id == seg_num || fusion_rec_iter->end_id == seg_num)
								//		ofs_fusion_candidate_file_remained_formated_2 << fusion_rec_iter->tostring(upper_bound_prev_iter2, sam_rec_iter, 2)<<endl;
								//	else
								//		ofs_fusion_candidate_file_remained_formated << fusion_rec_iter->tostring(upper_bound_prev_iter2, sam_rec_iter, 2)<<endl;

								remained = true;
							}
						}

						++matched_count2;
					}

					upper_bound_iter2 = upper_bound_prev_iter2;
				}

				if (!remained)
					ofs_fusion_candidate_fusion_single << fusion_rec_iter->tostring()<<endl;
			}

			sam_count = sam_file_handler.ReadNextTagAlign();

			sam_read_idx = sam_file_handler.read_id;

			fusion_count = fusion_file_handler.ReadNextTagAlign();

			fusion_read_idx = fusion_file_handler.read_id;
		}
		else if (sam_read_idx < fusion_read_idx)
		{
			vector<SamRec>::iterator sam_rec_iter;
			vector<SamRec> & sam_rec_vec = sam_file_handler.stored_tag_aligns;

			for (sam_rec_iter = sam_rec_vec.begin(); sam_rec_iter != sam_rec_vec.end(); ++sam_rec_iter)
			{
				ofs_fusion_candidate_normal_single << sam_rec_iter->tostring()<<endl;
			}

			sam_count = sam_file_handler.ReadNextTagAlign();

			sam_read_idx = sam_file_handler.read_id;			
		}
		else if (sam_read_idx > fusion_read_idx)
		{
			vector<FusionRec>::iterator fusion_rec_iter;
			vector<FusionRec> & fusion_rec_vec = fusion_file_handler.stored_tag_aligns;

			for (fusion_rec_iter = fusion_rec_vec.begin(); fusion_rec_iter !=fusion_rec_vec.end(); ++fusion_rec_iter)
				ofs_fusion_candidate_fusion_single << fusion_rec_iter->tostring()<<endl;

			
			fusion_count = fusion_file_handler.ReadNextTagAlign();

			fusion_read_idx = fusion_file_handler.read_id;
		}
		else
		{
			cerr << "what?"<<endl;
		}
	}

	return 0;

}


int main(int argc, char* argv[])
{
	const char* region_file = argv[1];

	const char* mapped_sam_file = argv[2];

	const char* fusion_aligned_file = argv[3];

	size_t ext = atoi(argv[4]);

	vector<PairedRegions> paired_regions;

	cout << "read regions" << endl;

	ReadRegions(region_file, paired_regions, ext);

	cout << max_dist<<endl;

	HASH_CHROM_PAIRED_REGIONS_PTR_VEC paired_regions_sorted1;

	HASH_CHROM_PAIRED_REGIONS_PTR_VEC paired_regions_sorted2;

	cout << "sort regions" << endl;

	SortRegion(paired_regions, paired_regions_sorted1, paired_regions_sorted2);

	FilterFusionCandidateByRegions(mapped_sam_file, fusion_aligned_file, paired_regions_sorted1, paired_regions_sorted2);
}
